/***************************************************************
 * Name:      wximagepanel.cpp
 * Purpose:   Side panel for PDF document priviewing.
 * Author:    Daniel Nell (daniel.nell@nellresearch.de)
 * Created:   2011-01-26
 * Copyright: Daniel Nell (www.nellresearch.de)
 * License:   GPL
 **************************************************************/

// Include precompiled headers.
#include "wx_pch.h"

// wxWidgets headers
#include <wx/config.h>
#include <wx/hashmap.h>
#include <wx/statbmp.h>

// Private headers
#include "wximagepanel.h"


//////////////////////////////////////////////////////////
// Class wxImagePanel
//
// Event table
//
BEGIN_EVENT_TABLE( wxImagePanel, wxScrolledWindow )
    EVT_MENU( wxExtThread::ID_EXIT, wxImagePanel::OnThreadExit )
END_EVENT_TABLE()


// Standard constructor
//
wxImagePanel::wxImagePanel( wxWindow *poParent, wxWindowID nId,
                            const wxPoint &oPos, const wxSize &oSize,
                            long nStyle, const wxString& strName )
: wxScrolledWindow( poParent, nId, oPos, oSize, nStyle, strName )
{
    // Empty the panel.
    Reset();

    // Reset window settings.
    wxConfigBase *poConfig = wxConfigBase::Get();
    poConfig->SetRecordDefaults();
    poConfig->SetPath( wxT( "/" )  );
    poConfig->SetPath( GetName() );

    // Reset bitmap and spacer size.
    m_nPanelPagesBitmapSize= poConfig->Read(  wxT( "PanelPagesBitmapSize" ),  160 );
    m_nPanelPagesBitmapSpace= poConfig->Read( wxT( "PanelPagesBitmapSpace" ),   6 );
}

// Virtual destructor
//
wxImagePanel::~wxImagePanel()
{
    // Save configuration data.
    wxConfigBase *poConfig= wxConfigBase::Get();

    poConfig->SetPath( wxT( "/" ) );
    poConfig->SetPath( GetName() );

    // Save bitmap and spacer size.
    poConfig->Write( wxT( "PanelPagesBitmapSize" ), (long)m_nPanelPagesBitmapSize );
    poConfig->Write( wxT( "PanelPagesBitmapSpace" ), (long)m_nPanelPagesBitmapSpace );
}

// Add images to the panel.
//
void wxImagePanel::AddImage( wxImage oImage )
{
    m_oImageHash[m_nPanelPagesCount]= oImage;
    wxImagePanelThread * poThread=new wxImagePanelThread( this, m_nPanelPagesCount );

    poThread->Create();             // Create the thread instance.
    poThread->SetPriority( 0 );     // Set the thread to the lowest possible priority.
    poThread->Run();                // Run the the thread.
    m_nPanelPagesCount++;
}

// Create a thumbnail from the corresponding image in another thread.
//
void wxImagePanel::OnThreadEntry( int nId )
{
    wxImage oImage= m_oImageHash[nId];

    double xfactor= (double)m_nPanelPagesBitmapSize
                    / (double)( ( oImage.GetWidth() > oImage.GetHeight() ) ? oImage.GetWidth() : oImage.GetHeight() );

    if( xfactor < 1.0 )
    {
        oImage.Rescale( (int)( (double)oImage.GetWidth()  * xfactor ),
                        (int)( (double)oImage.GetHeight() * xfactor ),
                        wxIMAGE_QUALITY_HIGH );
        m_oImageHash[nId]= oImage;
    }
}

// Handle the exit of another thread.
//
void wxImagePanel::OnThreadExit( wxCommandEvent& oEvent )
{
    int nId= oEvent.GetInt();

    wxImage oImage= m_oImageHash[nId];

    m_oImageHash.erase( nId );

    wxPoint oStartingPoint( m_nPanelPagesBitmapSpace,
                            m_nPanelPagesBitmapSpace + ( m_nPanelPagesBitmapSpace + m_nPanelPagesBitmapSize ) * nId );

    SetScrollbars( 1, 1,
                   m_nPanelPagesBitmapSpace + m_nPanelPagesBitmapSize + m_nPanelPagesBitmapSpace,
                   m_nPanelPagesBitmapSpace + ( m_nPanelPagesBitmapSpace + m_nPanelPagesBitmapSize ) * m_nPanelPagesCount );

    new wxStaticBitmap( this, wxID_ANY, wxBitmap( oImage ), oStartingPoint );
//    new wxBitmapButton( this, wxID_ANY, wxBitmap( oImage ), oStartingPoint );

#if wxCHECK_VERSION( 2, 9, 0 )
#else  // wxCHECK_VERSION( 2, 9, 0 )
    Scroll( 0, m_nPanelPagesBitmapSpace + ( m_nPanelPagesBitmapSpace + m_nPanelPagesBitmapSize ) * ( m_nPanelPagesCount - 1 ) );
#endif // wxCHECK_VERSION( 2, 9, 0 )
}

